import argparse
import os
import sys
import yaml

parser = argparse.ArgumentParser(description='Scrape R&D competitions and push the results to the SuiteCRM API.',
                                 prog='opportunity_scraper')
parser.add_argument('-c', '--config', dest='config',
                    default=f'{os.environ["HOME"]}/.config/opportunity_scraper/settings.yaml',
                    help='Location of config file')

try:
    file = open(parser.parse_args().config, 'r')
except Exception:
    print(f"Couldn't open config file {parser.parse_args().config}")
    sys.exit(1)

settings = yaml.load(file, Loader=yaml.FullLoader)



import json
from requests_oauthlib import OAuth2Session
from oauthlib.oauth2 import BackendApplicationClient
from selenium import webdriver
from .settings import settings
from . import govuk, ktn

__all__ = ['govuk', 'ktn']

def run():
    chrome_options = webdriver.ChromeOptions()
    chrome_options.add_argument('--headless')
    chrome_options.add_argument('--no-sandbox')
    chrome_options.add_argument("--start-maximized")
    chrome_options.add_argument("--window-size=1920x1080")
    browser = webdriver.Chrome(settings["browser"]["chromedriver"], options=chrome_options)
    govuk_opps = govuk.search(browser)
    ktn_opps = ktn.search(browser)
    browser.quit()

    client = BackendApplicationClient(client_id=settings["oauth"]["client_id"])
    session = OAuth2Session(client=client)
    session.fetch_token(token_url=settings["oauth"]["token_url"],
                        client_id=settings["oauth"]["client_id"],
                        client_secret=settings["oauth"]["client_secret"])

    for opp in govuk_opps + ktn_opps:
        req = session.post(f'{settings["suitecrm"]["api_url"]}/V8/module',
                           json={"data": opp.__dict__})
        title = opp.attributes["name"]
        if req.status_code == 201:
            print(f'Created opportunity: "{title}"')
        else:
            error = json.loads(req.text)["errors"]["detail"]
            print(f'Error creating opportunity: "{title}"')
            print(f'  "{error}"')
            


import re
from datetime import datetime
from selenium.common.exceptions import NoSuchElementException, TimeoutException
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait
from .opportunity import Opportunity
from .settings import settings

link_id_re = re.compile(r'/challenges/single/(\d+)')
open_re = re.compile(r'Launch of the Competition: (\d{1,2})([a-z]{2})? (of )?([a-zA-Z]{3,}) (20\d{2})')
close_re = re.compile(r'Deadline for applications: (\d{1,2})([a-z]{2})? (of )?([a-zA-Z]{3,}) (20\d{2})')

opp_headings = [
    'Background',
    'challenges',
    'Rewards and benefits',
    'Functional requirements',
    'Technical characteristics',
    'Operating conditions',
    'Deployment timescale',
    'Cost requirement and market opportunity',
    'Eligibility and assessment criteria',
    'IP and potential commercial route',

]

class KTN(Opportunity):

    def __init__(self, uri, title):
        super().__init__(uri, title)
        self.attributes["campaign_name"] = 'https://www.ktninnovationexchange.co.uk'
        self.attributes["account_name"] = settings["suitecrm"]["ktn_account_name"]
        self.attributes["account_id"] = settings["suitecrm"]["ktn_account_id"]

    def scrape(self, browser):
        if not super().scrape(browser):
            return
        content = browser.find_element_by_css_selector('.main-content .content .text')
        self.attributes["description"] = content.find_element_by_xpath('//h3[contains(text(), "Summary")]/following-sibling::p').text
        current_heading = 'background'
        current_text = None
        children = content.find_elements_by_xpath('//*[contains(@class, "description")]/*')
        for child in children:
            this_text = child.text.strip()
            if this_text in opp_headings:
                if current_heading:
                    self.attributes["description"] += f"\n\n{current_heading}\n\n{current_text}"
                    current_text = None
                current_heading = this_text
            else:
                if current_heading == 'Deployment timescale':
                    close_match = close_re.search(this_text)
                    if close_match:
                        self.attributes["date_closed"] = datetime.strftime(datetime.strptime(
                            f'{close_match.group(1)} {close_match.group(4)} {close_match.group(5)}', "%d %B %Y"), "%d/%m/%Y")
                    current_heading = None

                else:
                    if not current_text:
                        current_text = this_text
                    else:
                        current_text += this_text

        amount_match = re.findall(r'£[0-9,]+', self.attributes["description"])
        if amount_match:
            filtered = sorted(list(map(lambda x: int(re.sub(r'[^0-9]', '', x)), amount_match)), reverse=True)
            self.attributes["amount"] = filtered[0]

def login(browser):
    browser.get('https://www.ktninnovationexchange.co.uk/auth')
    form = browser.find_element_by_xpath('//*[@id="main-content"]/div/form')
    username = form.find_element_by_xpath('//*[@id="Email"]')
    password = form.find_element_by_xpath('//*[@id="Password"]')
    username.send_keys(settings["browser"]["ktn_username"])
    password.send_keys(settings["browser"]["ktn_password"])
    form.submit()

def search(browser, tried_auth=False):
    opportunities = []
    browser.get('https://www.ktninnovationexchange.co.uk/challenges')

    try:
        browser.find_element_by_xpath('/html/body[contains(@class, "logged-in")]')
    except NoSuchElementException:
        if tried_auth:
            raise
        login(browser)
        return search(browser, True)

    pages = browser.find_elements_by_xpath('//*[@id="js-challenge-pagination"]/li')
    current_page = 1

    while current_page <= len(pages) - 2:
        search_results = browser.find_elements_by_xpath('//*[@id="challenge-list-placeholder"]/div/div[contains(@class, "content")]')
        if not search_results:
            break
        for scrape in search_results:
            title = scrape.find_element_by_css_selector('.heading')
            link_id = scrape.find_element_by_css_selector('.btn')
            uri = link_id.get_attribute("href")
            id_match = link_id_re.search(uri)
            if id_match:
                opportunities.append(KTN(uri, title.text))
        current_page += 1
        try:
            next_page = browser.find_element_by_xpath(f'//*[@id="js-challenge-pagination"]/li/a[contains(text(), "{current_page}")]/..')
        except NoSuchElementException:
            break
        try:
            next_page.click()
            WebDriverWait(browser, 2).until(
                EC.staleness_of(link_id)
            )
        except TimeoutException:
            break

    print(f'==> Found {len(opportunities)} KTN competitions')
    for opp in opportunities:
        opp.scrape(browser)

    return opportunities



#!/usr/bin/env python3

"""Setup script"""

from setuptools import setup, find_packages

setup(
    name="opportunity_scraper",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    version="0.1.5",
    packages=find_packages(),
    install_requires=[
        'requests',
        'requests-oauthlib',
        'selenium',
    ],
    entry_points={
        'console_scripts': [
            'opportunity_scraper=opportunity_scraper:run',
        ]
    }
)



import uuid
from .settings import settings

class Opportunity:
    def __init__(self, uri, title):
        self.type = "Opportunities"
        self.id = uuid.uuid5(uuid.NAMESPACE_URL, uri).urn[9:]
        self.attributes = {
            "assigned_user_name": settings["suitecrm"]["assigned_user_name"],
            "assigned_user_id": settings["suitecrm"]["assigned_user_id"],
            "sales_stage": "Prospecting",
            "opportunity_type": "New Business",
            "lead_source": "Web Site",
            "next_step": uri,
            "name": title,
            "date_closed": None,
        }

    def scrape(self, browser):
        title = self.attributes["name"]
        print(f'Scraping {title}...')
        try:
            browser.get(self.attributes["next_step"])
        except Exception:
            print(f'Failed to fetch details for: {title}')
            return False
        return True



import re
from datetime import datetime
from selenium.common.exceptions import NoSuchElementException
from .opportunity import Opportunity
from .settings import settings

link_id_re = re.compile(r'/competition/(\d+)/overview')
date_re = re.compile(r'^.*: ')

def conform_date(dt):
    # The gov.uk opportunities site uses a variety of datetime formats,
    # some of them not easily manageable by strptime(). Conform them.
    dt = re.sub(r'^[a-zA-Z]+ (\d)', r'\1', dt)
    dt = re.sub(r'20(\d{2})\s*$', r'20\1 09:00am', dt)
    dt = re.sub(r'^(\d) ', r'0\1 ', dt)
    dt = re.sub(r' (\d):(\d{2})', r' 0\1:\2', dt)
    return datetime.strftime(datetime.strptime(dt, "%d %B %Y %I:%M%p"), "%d/%m/%Y")


class Govuk(Opportunity):

    def __init__(self, uri, title):
        super().__init__(uri, title)
        self.attributes["campaign_name"] = 'https://apply-for-innovation-funding.service.gov.uk'
        self.attributes["account_name"] = settings["suitecrm"]["govuk_account_name"]
        self.attributes["account_id"] = settings["suitecrm"]["govuk_account_id"]

    def scrape(self, browser):
        if not super().scrape(browser):
            return
        content = browser.find_element_by_css_selector('#main-content')
        self.attributes["description"] = content.find_element_by_css_selector('p:nth-child(2)').get_attribute('innerText')
        dates = content.find_element_by_css_selector('ul.govuk-list')
        date_closes = date_re.sub('', dates.find_element_by_css_selector('li:nth-child(2)').get_attribute('innerText'))
        self.attributes["date_closed"] = conform_date(date_closes)

        for section in browser.find_elements_by_css_selector('section.govuk-tabs__panel'):
            if section.get_attribute('id') == 'dates':
                self.attributes["description"] += "\n\nDates"
                dts = section.find_elements_by_css_selector('dt')
                dds = section.find_elements_by_css_selector('dd')
                for dt, dd in zip(dts, dds):
                    key = dt.get_attribute('innerText')
                    val = dd.get_attribute('innerText')
                    str_date = conform_date(key)
                    details = val.replace('\n', '')
                    self.attributes["description"] += f"\n> {str_date}: {details}"
            else:
                for row in section.find_elements_by_css_selector('.govuk-grid-row'):
                    heading = row.find_element_by_css_selector('.govuk-grid-column-one-third h2').get_attribute('innerText')
                    body = row.find_element_by_css_selector('.govuk-grid-column-two-thirds .govuk-body').get_attribute('innerText')
                    self.attributes["description"] += f"\n\n{heading}\n\n{body}"

        amount_match = re.findall(r'£[0-9,]+', self.attributes["description"])
        if amount_match:
            filtered = sorted(list(map(lambda x: int(re.sub(r'[^0-9]', '', x)),
                                       amount_match)), reverse=True)
            self.attributes["amount"] = filtered[0]


def search(browser):
    opportunities = []
    browser.get('https://apply-for-innovation-funding.service.gov.uk/competition/search')

    while True:
        search_results = browser.find_elements_by_css_selector('#main-content ul.govuk-list > li')
        for scrape in search_results:
            title_link = scrape.find_element_by_css_selector('h2.govuk-heading-m > a')
            uri = title_link.get_attribute("href")
            id_match = link_id_re.search(uri)
            if id_match:
                title = title_link.text
                opportunities.append(Govuk(uri, title))
        try:
            next_page = browser.find_element_by_css_selector('#main-content ul.pagination > li.next > a')
        except NoSuchElementException:
            break
        next_page.click()

    print(f'==> Found {len(opportunities)} gov.uk competitions')
    for opp in opportunities:
        opp.scrape(browser)

    return opportunities
